@@ -27,6 +27,7 @@
 #include "rdp.h"
 
 #include "info.h"
+#include "utils.h"
 #include "redirection.h"
 
 #include <freerdp/crypto/per.h>
@@ -34,7 +35,7 @@
 
 #define TAG FREERDP_TAG("core.rdp")
 
-const char* DATA_PDU_TYPE_STRINGS[80] = {
+static const char* DATA_PDU_TYPE_STRINGS[80] = {
 	"?",
 	"?",      /* 0x00 - 0x01 */
 	"Update", /* 0x02 */
@@ -102,10 +103,17 @@ const char* DATA_PDU_TYPE_STRINGS[80] = {
 	"?" /* 0x41 - 0x46 */
 };
 
+const char* data_pdu_type_to_string(UINT8 type)
+{
+	if (type > ARRAYSIZE(DATA_PDU_TYPE_STRINGS))
+		return "???";
+	return DATA_PDU_TYPE_STRINGS[type];
+}
+
 static BOOL rdp_read_flow_control_pdu(wStream* s, UINT16* type, UINT16* channel_id);
-static void rdp_write_share_control_header(wStream* s, UINT16 length, UINT16 type,
+static BOOL rdp_write_share_control_header(wStream* s, UINT16 length, UINT16 type,
                                            UINT16 channel_id);
-static void rdp_write_share_data_header(wStream* s, UINT16 length, BYTE type, UINT32 share_id);
+static BOOL rdp_write_share_data_header(wStream* s, UINT16 length, BYTE type, UINT32 share_id);
 
 /**
  * Read RDP Security Header.\n
@@ -192,13 +200,18 @@ BOOL rdp_read_share_control_header(wStream* s, UINT16* tpktLength, UINT16* remai
 	return TRUE;
 }
 
-void rdp_write_share_control_header(wStream* s, UINT16 length, UINT16 type, UINT16 channel_id)
+BOOL rdp_write_share_control_header(wStream* s, UINT16 length, UINT16 type, UINT16 channel_id)
 {
+	if (length < RDP_PACKET_HEADER_MAX_LENGTH)
+		return FALSE;
+	if (Stream_GetRemainingCapacity(s) < 6)
+		return FALSE;
 	length -= RDP_PACKET_HEADER_MAX_LENGTH;
 	/* Share Control Header */
 	Stream_Write_UINT16(s, length);      /* totalLength */
 	Stream_Write_UINT16(s, type | 0x10); /* pduType */
 	Stream_Write_UINT16(s, channel_id);  /* pduSource */
+	return TRUE;
 }
 
 BOOL rdp_read_share_data_header(wStream* s, UINT16* length, BYTE* type, UINT32* shareId,
@@ -218,11 +231,16 @@ BOOL rdp_read_share_data_header(wStream* s, UINT16* length, BYTE* type, UINT32*
 	return TRUE;
 }
 
-void rdp_write_share_data_header(wStream* s, UINT16 length, BYTE type, UINT32 share_id)
+BOOL rdp_write_share_data_header(wStream* s, UINT16 length, BYTE type, UINT32 share_id)
 {
-	length -= RDP_PACKET_HEADER_MAX_LENGTH;
-	length -= RDP_SHARE_CONTROL_HEADER_LENGTH;
-	length -= RDP_SHARE_DATA_HEADER_LENGTH;
+	const size_t headerLen = RDP_PACKET_HEADER_MAX_LENGTH + RDP_SHARE_CONTROL_HEADER_LENGTH +
+	                         RDP_SHARE_DATA_HEADER_LENGTH;
+	if (length < headerLen)
+		return FALSE;
+	length -= headerLen;
+	if (Stream_GetRemainingCapacity(s) < 12)
+		return FALSE;
+
 	/* Share Data Header */
 	Stream_Write_UINT32(s, share_id);  /* shareId (4 bytes) */
 	Stream_Write_UINT8(s, 0);          /* pad1 (1 byte) */
@@ -231,6 +249,7 @@ void rdp_write_share_data_header(wStream* s, UINT16 length, BYTE type, UINT32 sh
 	Stream_Write_UINT8(s, type);       /* pduType2, Data PDU Type (1 byte) */
 	Stream_Write_UINT8(s, 0);          /* compressedType (1 byte) */
 	Stream_Write_UINT16(s, 0);         /* compressedLength (2 bytes) */
+	return TRUE;
 }
 
 static BOOL rdp_security_stream_init(rdpRdp* rdp, wStream* s, BOOL sec_header)
@@ -396,7 +415,7 @@ BOOL rdp_read_header(rdpRdp* rdp, wStream* s, UINT16* length, UINT16* channelId)
 	{
 		if (code == X224_TPDU_DISCONNECT_REQUEST)
 		{
-			freerdp_abort_connect(rdp->instance);
+			utils_abort_connect(rdp->instance->context);
 			return TRUE;
 		}
 
@@ -453,7 +472,7 @@ BOOL rdp_read_header(rdpRdp* rdp, wStream* s, UINT16* length, UINT16* channelId)
 		}
 
 		WLog_DBG(TAG, "DisconnectProviderUltimatum: reason: %d", reason);
-		freerdp_abort_connect(rdp->instance);
+		utils_abort_connect(context);
 		EventArgsInit(&e, "freerdp");
 		e.code = 0;
 		PubSub_OnTerminate(context->pubSub, context, &e);
@@ -662,7 +681,8 @@ BOOL rdp_send_pdu(rdpRdp* rdp, wStream* s, UINT16 type, UINT16 channel_id)
 	sec_bytes = rdp_get_sec_bytes(rdp, 0);
 	sec_hold = Stream_GetPosition(s);
 	Stream_Seek(s, sec_bytes);
-	rdp_write_share_control_header(s, length - sec_bytes, type, channel_id);
+	if (!rdp_write_share_control_header(s, length - sec_bytes, type, channel_id))
+		return FALSE;
 	Stream_SetPosition(s, sec_hold);
 
 	if (!rdp_security_stream_out(rdp, s, length, 0, &pad))
@@ -698,8 +718,10 @@ BOOL rdp_send_data_pdu(rdpRdp* rdp, wStream* s, BYTE type, UINT16 channel_id)
 	sec_bytes = rdp_get_sec_bytes(rdp, 0);
 	sec_hold = Stream_GetPosition(s);
 	Stream_Seek(s, sec_bytes);
-	rdp_write_share_control_header(s, length - sec_bytes, PDU_TYPE_DATA, channel_id);
-	rdp_write_share_data_header(s, length - sec_bytes, type, rdp->settings->ShareId);
+	if (!rdp_write_share_control_header(s, length - sec_bytes, PDU_TYPE_DATA, channel_id))
+		goto fail;
+	if (!rdp_write_share_data_header(s, length - sec_bytes, type, rdp->settings->ShareId))
+		goto fail;
 	Stream_SetPosition(s, sec_hold);
 
 	if (!rdp_security_stream_out(rdp, s, length, 0, &pad))
@@ -920,8 +942,7 @@ int rdp_recv_data_pdu(rdpRdp* rdp, wStream* s)
 	}
 
 	WLog_DBG(TAG, "recv %s Data PDU (0x%02" PRIX8 "), length: %" PRIu16 "",
-	         type < ARRAYSIZE(DATA_PDU_TYPE_STRINGS) ? DATA_PDU_TYPE_STRINGS[type] : "???", type,
-	         length);
+	         data_pdu_type_to_string(type), type, length);
 
 	switch (type)
 	{
@@ -1250,7 +1271,7 @@ BOOL rdp_decrypt(rdpRdp* rdp, wStream* s, UINT16* pLength, UINT16 securityFlags)
 	return TRUE;
 }
 
-static const char* pdu_type_to_str(UINT16 pduType)
+const char* pdu_type_to_str(UINT16 pduType)
 {
 	static char buffer[1024] = { 0 };
 	switch (pduType)
@@ -1628,12 +1649,7 @@ int rdp_recv_callback(rdpTransport* transport, wStream* s, void* extra)
 
 			if ((status >= 0) && (rdp->finalize_sc_pdus == FINALIZE_SC_COMPLETE))
 			{
-				ActivatedEventArgs activatedEvent;
-				rdpContext* context = rdp->context;
 				rdp_client_transition_to_state(rdp, CONNECTION_STATE_ACTIVE);
-				EventArgsInit(&activatedEvent, "libfreerdp");
-				activatedEvent.firstActivation = !rdp->deactivation_reactivation;
-				PubSub_OnActivated(context->pubSub, context, &activatedEvent);
 				return 2;
 			}
 
