@@ -244,32 +244,16 @@ static void settings_load_hkey_local_machine(rdpSettings* settings)
 
 static BOOL settings_get_computer_name(rdpSettings* settings)
 {
-	DWORD nSize = 0;
-	CHAR* computerName;
-
-	if (GetComputerNameExA(ComputerNameNetBIOS, NULL, &nSize) || GetLastError() != ERROR_MORE_DATA)
-		return FALSE;
-
-	computerName = calloc(nSize, sizeof(CHAR));
-
-	if (!computerName)
-		return FALSE;
+	CHAR computerName[256];
+	DWORD nSize = sizeof(computerName);
 
 	if (!GetComputerNameExA(ComputerNameNetBIOS, computerName, &nSize))
-	{
-		free(computerName);
 		return FALSE;
-	}
 
 	if (nSize > MAX_COMPUTERNAME_LENGTH)
 		computerName[MAX_COMPUTERNAME_LENGTH] = '\0';
 
-	settings->ComputerName = computerName;
-
-	if (!settings->ComputerName)
-		return FALSE;
-
-	return TRUE;
+	return freerdp_settings_set_string(settings, FreeRDP_ComputerName, computerName);
 }
 
 BOOL freerdp_settings_set_default_order_support(rdpSettings* settings)
@@ -367,6 +351,7 @@ rdpSettings* freerdp_settings_new(DWORD flags)
 	settings->DisableMenuAnims = TRUE;
 	settings->DisableThemes = FALSE;
 	settings->ConnectionType = CONNECTION_TYPE_LAN;
+	settings->NetworkAutoDetect = TRUE;
 	settings->EncryptionMethods = ENCRYPTION_METHOD_NONE;
 	settings->EncryptionLevel = ENCRYPTION_LEVEL_NONE;
 	settings->FIPSMode = FALSE;
@@ -436,7 +421,7 @@ rdpSettings* freerdp_settings_new(DWORD flags)
 	settings->AllowCacheWaitingList = TRUE;
 	settings->BitmapCacheV2NumCells = 5;
 	settings->BitmapCacheV2CellInfo =
-	    (BITMAP_CACHE_V2_CELL_INFO*)malloc(sizeof(BITMAP_CACHE_V2_CELL_INFO) * 6);
+	    (BITMAP_CACHE_V2_CELL_INFO*)calloc(6, sizeof(BITMAP_CACHE_V2_CELL_INFO));
 
 	if (!settings->BitmapCacheV2CellInfo)
 		goto out_fail;
@@ -455,12 +440,12 @@ rdpSettings* freerdp_settings_new(DWORD flags)
 	settings->RefreshRect = TRUE;
 	settings->SuppressOutput = TRUE;
 	settings->GlyphSupportLevel = GLYPH_SUPPORT_NONE;
-	settings->GlyphCache = malloc(sizeof(GLYPH_CACHE_DEFINITION) * 10);
+	settings->GlyphCache = calloc(10, sizeof(GLYPH_CACHE_DEFINITION));
 
 	if (!settings->GlyphCache)
 		goto out_fail;
 
-	settings->FragCache = malloc(sizeof(GLYPH_CACHE_DEFINITION));
+	settings->FragCache = calloc(1, sizeof(GLYPH_CACHE_DEFINITION));
 
 	if (!settings->FragCache)
 		goto out_fail;
@@ -543,30 +528,32 @@ rdpSettings* freerdp_settings_new(DWORD flags)
 
 	settings->DeviceArraySize = 16;
 	settings->DeviceArray =
-	    (RDPDR_DEVICE**)calloc(1, sizeof(RDPDR_DEVICE*) * settings->DeviceArraySize);
+	    (RDPDR_DEVICE**)calloc(settings->DeviceArraySize, sizeof(RDPDR_DEVICE*));
 
 	if (!settings->DeviceArray)
 		goto out_fail;
 
 	settings->StaticChannelArraySize = 16;
 	settings->StaticChannelArray =
-	    (ADDIN_ARGV**)calloc(1, sizeof(ADDIN_ARGV*) * settings->StaticChannelArraySize);
+	    (ADDIN_ARGV**)calloc(settings->StaticChannelArraySize, sizeof(ADDIN_ARGV*));
 
 	if (!settings->StaticChannelArray)
 		goto out_fail;
 
 	settings->DynamicChannelArraySize = 16;
 	settings->DynamicChannelArray =
-	    (ADDIN_ARGV**)calloc(1, sizeof(ADDIN_ARGV*) * settings->DynamicChannelArraySize);
+	    (ADDIN_ARGV**)calloc(settings->DynamicChannelArraySize, sizeof(ADDIN_ARGV*));
 
 	if (!settings->DynamicChannelArray)
 		goto out_fail;
 
-	settings->TcpKeepAlive = TRUE;
-	settings->TcpKeepAliveRetries = 3;
-	settings->TcpKeepAliveDelay = 5;
-	settings->TcpKeepAliveInterval = 2;
-	settings->TcpAckTimeout = 9000;
+	if (!freerdp_settings_set_bool(settings, FreeRDP_TcpKeepAlive, TRUE) ||
+	    !freerdp_settings_set_uint32(settings, FreeRDP_TcpKeepAliveRetries, 3) ||
+	    !freerdp_settings_set_uint32(settings, FreeRDP_TcpKeepAliveDelay, 5) ||
+	    !freerdp_settings_set_uint32(settings, FreeRDP_TcpKeepAliveInterval, 2) ||
+	    !freerdp_settings_set_uint32(settings, FreeRDP_TcpAckTimeout, 9000) ||
+	    !freerdp_settings_set_uint32(settings, FreeRDP_TcpConnectTimeout, 15000))
+		goto out_fail;
 
 	if (!settings->ServerMode)
 	{
@@ -610,6 +597,7 @@ rdpSettings* freerdp_settings_new(DWORD flags)
 	settings_load_hkey_local_machine(settings);
 
 	settings->ActionScript = _strdup("~/.config/freerdp/action.sh");
+	settings->XSelectionAtom = NULL;
 	settings->SmartcardLogon = FALSE;
 	settings->TlsSecLevel = 1;
 	settings->OrderSupport = calloc(1, 32);
@@ -656,6 +644,8 @@ static void freerdp_settings_free_internal(rdpSettings* settings)
 	/* Extensions */
 	free(settings->ActionScript);
 	settings->ActionScript = NULL;
+	free(settings->XSelectionAtom);
+	settings->XSelectionAtom = NULL;
 
 	/* Free all strings, set other pointers NULL */
 	freerdp_settings_free_keys(settings, TRUE);
@@ -697,7 +687,7 @@ static BOOL freerdp_settings_int_buffer_copy(rdpSettings* _settings, const rdpSe
 		if (!_settings->ServerRandom)
 			goto out_fail;
 
-		CopyMemory(_settings->ServerRandom, settings->ServerRandom, _settings->ServerRandomLength);
+		CopyMemory(_settings->ServerRandom, settings->ServerRandom, settings->ServerRandomLength);
 		_settings->ServerRandomLength = settings->ServerRandomLength;
 	}
 
@@ -708,10 +698,22 @@ static BOOL freerdp_settings_int_buffer_copy(rdpSettings* _settings, const rdpSe
 		if (!_settings->ClientRandom)
 			goto out_fail;
 
-		CopyMemory(_settings->ClientRandom, settings->ClientRandom, _settings->ClientRandomLength);
+		CopyMemory(_settings->ClientRandom, settings->ClientRandom, settings->ClientRandomLength);
 		_settings->ClientRandomLength = settings->ClientRandomLength;
 	}
 
+	if (settings->ServerCertificateLength)
+	{
+		_settings->ServerCertificate = (BYTE*)malloc(settings->ServerCertificateLength);
+
+		if (!_settings->ServerCertificate)
+			goto out_fail;
+
+		CopyMemory(_settings->ServerCertificate, settings->ServerCertificate,
+		           settings->ServerCertificateLength);
+		_settings->ServerCertificateLength = settings->ServerCertificateLength;
+	}
+
 	if (settings->RdpServerCertificate)
 	{
 		_settings->RdpServerCertificate = certificate_clone(settings->RdpServerCertificate);
@@ -983,6 +985,8 @@ static BOOL freerdp_settings_int_buffer_copy(rdpSettings* _settings, const rdpSe
 
 	if (settings->ActionScript)
 		_settings->ActionScript = _strdup(settings->ActionScript);
+	if (settings->XSelectionAtom)
+		_settings->XSelectionAtom = _strdup(settings->XSelectionAtom);
 	rc = TRUE;
 out_fail:
 	return rc;
@@ -1024,6 +1028,7 @@ BOOL freerdp_settings_copy(rdpSettings* _settings, const rdpSettings* settings)
 	_settings->StaticChannelArray = NULL;
 	_settings->DynamicChannelArray = NULL;
 	_settings->ActionScript = NULL;
+	_settings->XSelectionAtom = NULL;
 	if (!rc)
 		goto out_fail;
 
